#pragma rtGlobals=1		// Use modern global access method.

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Jan,13,2009																									//
//Ver 1.0																											//
//This is a simple utility for download long period of history data from wunderground.com.									//
//The function require EasyHttp Xop.																				//
//The main function take in input the station ID, station type start date and stop date to download.							//		
//Station type must be 1 if the weather station where get the data is an airport (Metar data).								//
//Station type must be NOT 1 if the weather station where get the data is a Personal Weather Station (PSW)				//
//Station ID is the a string code that identify the weather station. 														//
//In case of Airport the Station ID correspond to the ICAO code of the airport. A list of icao codes can be found here: 			//
//http://www.airport-technology.com/icao-codes/  or here:																//
//http://en.wikipedia.org/wiki/International_Civil_Aviation_Organization_airport_code										//
//In case of PSW, a list of available stations can be retrived on http://www.wunderground.com/weatherstation/ListStations.asp	//
//Before start the download is preferible to check on the website if the data is available. 									//
//																												//
//																												//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


Function DownLoadWUData(Station_IDStr, YYStart, MMStart, DDStart, YYStop, MMStop, DDStop,StationType)

	String Station_IDStr		//Station Code
	Variable YYStart, MMStart, DDStart, YYStop, MMStop, DDStop, StationType // Start and stop date to download . StationType=1 means Metar data format .

	SetDataFolder root:DWU
	
	SVar Link01		// URL block
	SVar Link02		//
	SVar Link03		//
	
	SVar ListOfMetarWvStr			//List of waves to use in case of data from airport
	SVar ListOfMeteoWvStr			//List of waves to use in case of data from PSW
	
	NewDataFolder/O/S root:DWU:$Station_IDStr

	String ListOfWv = ListOfMeteoWvStr
	if(StationType ==1)
		ListOfWv = ListOfMetarWvStr
	EndIf
	
	String TempStr
	Variable CntD, CntM, CntY, Row, i=1, DVar,  Cnt		//temporany variables
	for(Cnt=0; cnt<=11;cnt+=1)	//it create the waves where store the data
		If(StringMatch(StringFromList(Cnt,ListOfWv,","), "*TxtWv"))
			Make/T/O/N=(1) $StringFromList(Cnt,ListOfWv,",")
		Else
			Make/D/O/N=(1) $StringFromList(Cnt,ListOfWv,",")
		EndIf
	EndFor
	Make/D/O/N=(1) timeWv		//date wave
	CntM = MMStart
	For(CntY=YYStart; CntY<=YYStop; CntY+=1)
		For(CntM=MMStart; CntM<=12; CntM+=1)
		DVar = (CntM == 4 || CntM==6 || CntM==9 || CntM==11) ? 30 : 31
		DVar = CntM == 2 ? 28 : DVar
			For(CntD=DDStart; CntD<=DVar;CntD+=1)
				Switch(StationType)
				case 1:
					easyhttp Link01+Station_IDStr+"/"+num2str(CntY)+"/"+num2str(CntM)+"/"+num2str(CntD)+Link02		//build the daily proper URL and get the data from WUnderground.com (metar data from airport station)
				break
				Default:
					easyhttp Link03+Station_IDStr+"&month="+num2str(CntM)+"&day="+num2str(CntD)+"&year="+num2str(CntY)+"&format=1"	//build the daily proper URL and get the data from WUnderground.com (Data from PWS)
				break
				EndSwitch
			
				If(V_Flag)
					DoAlert 1, "Problem Downloading data on"+" " + Num2Str(CntM) + "-" + Num2Str(CntD) + "-" + Num2Str(CntY)+"\nDo you want to continue the download?"
					If(V_flag==2)
						Abort
					EndIf
					Print "Problem Downloading data on"+" " + Num2Str(CntM) + "-" + Num2Str(CntD) + "-" + Num2Str(CntY)		//Print the date when any problem happen downloading the data	
				EndIf
				do
					TempStr = StringFromList(i, S_GetHttp, ">") 	//Get a single row from the daily data report
					i+=1
					If(StringMatch(TempStr,"*--*"))		//It break the loop at the end of the daily data report ("--" is present on the last row of the daily report page)
						break
					endif
					If(!StringMatch(StringFromList(1,TempStr,","), "*.*")	)	//it is "working around" to check whenever "aaxx" data are present (instead of  Metar data) and skip that, I can no found cleverest way to do that.
						Continue
					EndIf
			
					for(Cnt=0; cnt<=11;cnt+=1)		//parse a single row and put the data on the proper waves
						If(StringMatch(StringFromList(Cnt,ListOfWv,","), "*TxtWv"))
							Wave/T TxtWv =$StringFromList(Cnt,ListOfWv,",")
							InsertPoints Row, 1, TxtWv
							TxtWv[row] = StringFromList(Cnt,TempStr,",")
						Else
							Wave NumWv =$StringFromList(Cnt,ListOfWv,",")
							InsertPoints Row, 1, NumWv
							NumWv[row] = Str2Num(StringFromList(Cnt,TempStr,","))
						EndIf
					EndFor
						
					InsertPoints Row, 1, TimeWv																	//
					Wave/T TimeTxtWv																			//		
					TempStr = Num2Str(CntY)+"-"+Num2Str(CntM)+"-"+Num2Str(CntD)+" "+TimeTxtWv[row]				//
					TimeWv[row] = (StationType==1) ? GetIgorTime(TempStr) : GetIgorTime(TimeTxtWv[row])				//build a "igor time wave"
					Row+=1
				while(1)
				i=1
				if(CntM == MMStop && CntD == DDStop && CntY == YYStop)		//check the stop date to download and stop the function
					return 1
				EndIf
			EndFor
		DDStart = 1
		EndFor
	MMStart = 1
	EndFor

End

Function GetIgorTime(str)		//parse date/time in string format and return date/time in igor format 
	String str			// Format: "yyyy-mm-dd hh:mm:ss" or "yyyy-mm-dd hh:mm"
 	Variable day, month, year, hour, minute,second
	sscanf str, "%4d-%2d-%2d %d:%d:%d", year, month, day, hour, minute, second
	if(stringmatch(str, "*PM*"))
		if(hour!=12)
			hour+=12
		EndIf
	ElseIf(stringmatch(str, "*AM*"))
		if(hour==12)
			hour-=12
		EndIf
	EndIf
	Variable dt			
	dt = (date2secs(year, month, day)) +  3600*hour + 60*minute + second
	return dt
End

Function ButtClosePr(ctrlName) : ButtonControl //"CLOSE" Button
	String ctrlName
	DoWindow/K DWUPanel
End

Function ButtSaveDwu(ctrlName) : ButtonControl		//"SAVE" Button
	String ctrlName
	SVar StationIDStr = root:dwu:StationIDStr
	SetDataFolder root:dwu:$StationIDStr
	String list
	list = WaveList("*", ";", "")
	Save/J/W/I/B list as "Meteo data.txt"
	Button Butt_Save, disable=1
	SetDataFolder root:
End

Function PopMenuProc01(ctrlName,popNum,popStr) : PopupMenuControl	//Set Station Type
	String ctrlName
	Variable popNum
	String popStr
	NVar StType = Root:Dwu:StType
	StType = popNum
End


Function ButtDownPr(ctrlName) : ButtonControl		//Check Station ID/Station Type and Start the download
	String ctrlName
	SetDataFolder root:DWU
	nVar YStartVar, MStartVar, DStartVar, YStopVar, MStopVar, DStopVar, StType
	sVar StationIDStr
	if(StType==0||StrLen(StationIDStr)<1)
		Doalert 0, "Please selct Station Type and enter a valid station ID before continue ."
		SetDataFolder root:
		abort
	EndIf
	if(StType==1&&StrLen(StationIDStr)!=4)
		Doalert 0, "Station ID lenght should be 4 characters long in case of airport (ICAO Code). Check Station ID/Type before continue."
		SetDataFolder root: 
		abort
	EndIf
	If(stringmatch(StationIDStr, "* *"))
		Doalert 0, "Space character present on Station ID. The Station Id would not have to contain space character. Check Station ID before continue. "
		SetDataFolder root: 
		abort
	EndIf
	DownLoadWUData(StationIDStr, YStartVar, MStartVar, DStartVar, YStopVar, MStopVar, DStopVar, StType)
	Button Butt_Save, disable=0
	SetDataFolder root:
	
End

Window DWUPanel() : Panel
	PauseUpdate; Silent 1		// building window...
	NewPanel /W=(516,109,848,400) as "Download data from WUnderground.com"
	SetVariable setvar0,pos={8,28},size={90,18},proc=SetVarProc,title="Year"
	SetVariable setvar0,value= root:DWU:YStartVar
	SetVariable setvar1,pos={118,28},size={70,18},title="Month"
	SetVariable setvar1,value= root:DWU:MStartVar
	SetVariable setvar2,pos={218,28},size={80,18},title="Day"
	SetVariable setvar2,value= root:DWU:DStartVar
	SetVariable setvar5,pos={8,88},size={90,18},title="Year"
	SetVariable setvar5,value= root:DWU:YStopVar
	SetVariable setvar6,pos={118,88},size={70,18},title="Month"
	SetVariable setvar6,value= root:DWU:MStopVar
	SetVariable setvar7,pos={218,88},size={80,18},title="Day"
	SetVariable setvar7,value= root:DWU:DStopVar
	Button Butt_Down,pos={11,211},size={150,26},proc=ButtDownPr,title="Start Download"
	Button Butt_Down,fSize=20
	Button Butt_closepan,pos={173,212},size={150,26},proc=ButtClosePr,title="Close Panel"
	Button Butt_closepan,fSize=20
	Button Butt_Save,pos={12,243},size={310,25},proc=ButtSaveDwu,disable=1,title="Save Last Download data"
	Button Butt_Save,fSize=20
	SetVariable setstr5,pos={6,170},size={200,18},title="Station ID"
	SetVariable setstr5,value= root:DWU:StationIDStr
	PopupMenu popup1,pos={3,135},size={210,23},proc=PopMenuProc01,title="Station Type"
	PopupMenu popup1,mode=3,popvalue="Select type of Station",value= #"\"Airport;Personal Weather Station (PSW)\""
	GroupBox group0,pos={3,8},size={304,49},title="Start period"
	GroupBox group1,pos={3,63},size={304,49},title="Stop period"
	setdatafolder root:
EndMacro

Function InitDWUPan()
	NewDataFolder/O/S DWU
	
	String/G Link01 = "http://www.wunderground.com/history/airport/"		//first part of the URL to build
	String/G Link02 = "/DailyHistory.html?&format=1"	//Last part of the URL to build
	String/G Link03 = "http://www.wunderground.com/weatherstation/WXDailyHistory.asp?ID="
	
	//List of waves to use in case of data from airport
	String/G ListOfMetarWvStr = "TimeTxtWv,TemperatureWv,DewPointWv,HumidityWv,PressureWv,VisibilityWv,WindDirTxtWv,WindSpeedWv,GustSpeedWv,PrecipitationWv,EventTxtWv,ConditionTxtWv"//List of the waves where store that data
	String/G ListOfMeteoWvStr = "TimeTxtWv,TemperatureWv,DewpointWv,PressureWv,WindDirectionTxtWv,WindDirectionDegreesWv,WindSpeedWv,WindSpeedGustWv,HumidityWv,HourlyPrecipWv,ConditionsTxtWv,CloudsTxtWv,dailyrainWv,SoftwareTypeWv"
	
	String/G StationIDStr = ""		//Station ID
	String TempDate = Secs2Date(DateTime,-1)
	Variable/G YStartVar=Str2Num(TempDate[6,9])		
	Variable/G MStartVar=Str2Num(TempDate[3,4])			
	Variable/G DStartVar=(Str2Num(TempDate[0,1]))-1		
	Variable/G YStopVar=YStartVar
	Variable/G MStopVar=MStartVar
	Variable/G DStopVar=DStartVar+1
	Variable/G StType=0
	if(stringmatch(WinList("HuTongPan","",""), "HuTongPan" ) ==1)				
		dowindow/F DWUPanel
	else
		Execute/Q "DWUPanel()"
	endif
	SetDataFolder root:
end

Menu "Tools"
	"Download Data from WeatherUnderground.com", /Q, InitDWUPan() 
End